using OSMDatastructure;
using OSMDatastructure.Graph;

namespace OSMImporter
{
    public class RegionManager
    {
        private string workingDirectory { get; }
        private readonly Dictionary<ulong, Region> _regions = new();

        public RegionManager(string workingDirectory)
        {
            this.workingDirectory = workingDirectory;
        }
        
        /// <summary>
        /// Checks wether the Region is already loaded and returns the Region, or tries to load the Region from file in workingDirectory
        /// </summary>
        /// <param name="coordinates">Coordinates of the Region (or within the Region) to load</param>
        /// <returns>The Region at the specified Coordinates containing Nodes and Connections</returns>
        /// <exception cref="FileNotFoundException">If the Regionfile can not be found.</exception>
        public Region GetRegion(Coordinates coordinates)
        {
            if(_regions.ContainsKey(Coordinates.GetRegionHashCode(coordinates)))
                return _regions[Coordinates.GetRegionHashCode(coordinates)];
            else
            {
                Region loadedRegion = LoadRegion(coordinates);
                _regions.Add(loadedRegion.regionHash, value: loadedRegion);
                return loadedRegion;
            }
        }

        public Region[] GetAllRegions()
        {
            return this._regions.Values.ToArray();
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="coordinates">Coordinates of the Region (or within the Region) to load</param>
        /// <returns>The Region at the specified Coordinates containing Nodes and Connections</returns>
        /// <exception cref="FileNotFoundException">If the Regionfile can not be found.</exception>
        private Region LoadRegion(Coordinates coordinates)
        {
            throw new NotImplementedException();
        }

        public OsmNode? GetNode(Coordinates coordinates)
        {
            Region regionWithNode = GetRegion(coordinates);
            return regionWithNode.GetNode(coordinates);
        }
    }
}